/**
 * @file       At86rf215_regs.hpp
 * @author     Pere Tuset-Peiro     (peretuset@openmote.com)
 * @version    v0.1
 * @date       May, 2016
 * @brief
 *
 * @copyright  Copyright 2016, OpenMote Technologies, S.L.
 *             This file is licensed under the GNU General Public License v2.
 */

#ifndef AT86RF215_REGS_HPP_
#define AT86RF215_REGS_HPP_

#define READ_CMD              ( 0x00 )
#define WRITE_CMD             ( 0x80 )

#define RF_CMD_NOP            ( 0x00 )
#define RF_CMD_SLEEP          ( 0x01 )
#define RF_CMD_TRXOFF         ( 0x02 )
#define RF_CMD_TXPREP         ( 0x03 )
#define RF_CMD_TX             ( 0x04 )
#define RF_CMD_RX             ( 0x05 )
#define RF_CMD_RESET          ( 0x07 )

#define RF_STATE_TRXOFF       ( 0x02 )
#define RF_STATE_TXPREP       ( 0x03 )
#define RF_STATE_TX           ( 0x04 )
#define RF_STATE_RX           ( 0x05 )
#define RF_STATE_TRANS        ( 0x06 )
#define RF_STATE_RESET        ( 0x07 )

#define PN_215                ( 0x34 )
#define PN_215_IQ             ( 0x35 )
#define PN_215_M              ( 0x36 )
#define VN_1_1                ( 0x01 )
#define VN_1_3                ( 0x03 )

#define RF09_IRQS             ( 0x0000 )
#define RF24_IRQS             ( 0x0001 )
#define BBC0_IRQS             ( 0x0002 )
#define BBC1_IRQS             ( 0x0003 )
#define RF_RST                ( 0x0005 )
#define RF_CFG                ( 0x0006 )
#define RF_CLKO               ( 0x0007 )
#define RF_BMDVC              ( 0x0008 )
#define RF_XOC                ( 0x0009 )
#define RF_IQIFC0             ( 0x000A )
#define RF_IQIFC1             ( 0x000B )
#define RF_IQIFC2             ( 0x000C )
#define RF_PN                 ( 0x000D )
#define RF_VN                 ( 0x000E )

#define RF09_BASE             ( 0x0100 )
#define RF24_BASE             ( 0x0200 )
#define BBC0_BASE             ( 0x0300 )
#define BBC1_BASE             ( 0x0400 )
#define BBC0_FB_BASE          ( 0x2000 )
#define BBC1_FB_BASE          ( 0x3000 )

#define RFn_IRQM              ( 0x0000 )
#define RFn_AUXS              ( 0x0001 )
#define RFn_STATE             ( 0x0002 )
#define RFn_CMD               ( 0x0003 )
#define RFn_CS                ( 0x0004 )
#define RFn_CCF0L             ( 0x0005 )
#define RFn_CCF0H             ( 0x0006 )
#define RFn_CNL               ( 0x0007 )
#define RFn_CNM               ( 0x0008 )
#define RFn_RXBWC             ( 0x0009 )
#define RFn_RXDFE             ( 0x000A )
#define RFn_AGCC              ( 0x000B )
#define RFn_AGCS              ( 0x000C )
#define RFn_RSSI              ( 0x000D )
#define RFn_EDC               ( 0x000E )
#define RFn_EDD               ( 0x000F )
#define RFn_EDV               ( 0x0010 )
#define RFn_RNDV              ( 0x0011 )
#define RFn_TXCUTC            ( 0x0012 )
#define RFn_TXDFE             ( 0x0013 )
#define RFn_PAC               ( 0x0014 )
#define RFn_PADFE             ( 0x0016 )
#define RFn_PLL               ( 0x0021 )
#define RFn_PLLCF             ( 0x0022 )
#define RFn_TXCI              ( 0x0025 )
#define RFn_TXCQ              ( 0x0026 )
#define RFn_TXDACI            ( 0x0027 )
#define RFn_TXDACQ            ( 0x0028 )

#define BBCn_IRQM             ( 0x0000 )
#define BBCn_PC               ( 0x0001 )
#define BBCn_PS               ( 0x0002 )
#define BBCn_RXFLL            ( 0x0004 )
#define BBCn_RXFLH            ( 0x0005 )
#define BBCn_TXFLL            ( 0x0006 )
#define BBCn_TXFLH            ( 0x0007 )
#define BBCn_FBLL             ( 0x0008 )
#define BBCn_FBLH             ( 0x0009 )
#define BBCn_FBLIL            ( 0x000A )
#define BBCn_FBLIH            ( 0x000B )
#define BBCn_OFDMPHRTX        ( 0x000C )
#define BBCn_OFDMPHRRX        ( 0x000D )
#define BBCn_OFDMC            ( 0x000E )
#define BBCn_OFDMSW           ( 0x000F )
#define BBCn_OQPSKC0          ( 0x0010 )
#define BBCn_OQPSKC1          ( 0x0011 )
#define BBCn_OQPSKC2          ( 0x0012 )
#define BBCn_OQPSKC3          ( 0x0013 )
#define BBCn_OQPSKPHRTX       ( 0x0014 )
#define BBCn_OQPSKPHRRX       ( 0x0015 )
#define BBCn_AFC0             ( 0x0020 )
#define BBCn_AFC1             ( 0x0021 )
#define BBCn_AFFTM            ( 0x0022 )
#define BBCn_AFFVM            ( 0x0023 )
#define BBCn_AFS              ( 0x0024 )
#define BBCn_MACEA0           ( 0x0025 )
#define BBCn_MACEA1           ( 0x0026 )
#define BBCn_MACEA2           ( 0x0027 )
#define BBCn_MACEA3           ( 0x0028 )
#define BBCn_MACEA4           ( 0x0029 )
#define BBCn_MACEA5           ( 0x002A )
#define BBCn_MACEA6           ( 0x002B )
#define BBCn_MACEA7           ( 0x002C )
#define BBCn_MACPID0F0        ( 0x002D )
#define BBCn_MACPID1F0        ( 0x002E )
#define BBCn_MACSHA0F0        ( 0x002F )
#define BBCn_MACSHA1F0        ( 0x0030 )
#define BBCn_MACPID0F1        ( 0x0031 )
#define BBCn_MACPID1F1        ( 0x0032 )
#define BBCn_MACSHA0F1        ( 0x0033 )
#define BBCn_MACSHA1F1        ( 0x0034 )
#define BBCn_MACPID0F2        ( 0x0035 )
#define BBCn_MACPID1F2        ( 0x0036 )
#define BBCn_MACSHA0F2        ( 0x0037 )
#define BBCn_MACSHA1F2        ( 0x0038 )
#define BBCn_MACPID0F3        ( 0x0039 )
#define BBCn_MACPID1F3        ( 0x003A )
#define BBCn_MACSHA0F3        ( 0x003B )
#define BBCn_MACSHA1F3        ( 0x003C )
#define BBCn_AMCS             ( 0x0040 )
#define BBCn_AMEDT            ( 0x0041 )
#define BBCn_AMAACKPD         ( 0x0042 )
#define BBCn_AMAACKTL         ( 0x0043 )
#define BBCn_AMAACKTH         ( 0x0044 )
#define BBCn_FSKC0            ( 0x0060 )
#define BBCn_FSKC1            ( 0x0061 )
#define BBCn_FSKC2            ( 0x0062 )
#define BBCn_FSKC3            ( 0x0063 )
#define BBCn_FSKC4            ( 0x0064 )
#define BBCn_FSKPLL           ( 0x0065 )
#define BBCn_FSKSFD0L         ( 0x0066 )
#define BBCn_FSKSFD0H         ( 0x0067 )
#define BBCn_FSKSFD1L         ( 0x0068 )
#define BBCn_FSKSFD1H         ( 0x0069 )
#define BBCn_FSKPHRTX         ( 0x006A )
#define BBCn_FSKPHRRX         ( 0x006B )
#define BBCn_FSKRPC           ( 0x006C )
#define BBCn_FSKRPCONT        ( 0x006D )
#define BBCn_FSKRPCOFFT       ( 0x006E )
#define BBCn_FSKRRXFLL        ( 0x0070 )
#define BBCn_FSKRRXFLH        ( 0x0071 )
#define BBCn_FSKDM            ( 0x0072 )
#define BBCn_FSKPE0           ( 0x0073 )
#define BBCn_FSKPE1           ( 0x0074 )
#define BBCn_FSKPE2           ( 0x0075 )
#define BBCn_PMUC             ( 0x0080 )
#define BBCn_PMUVAL           ( 0x0081 )
#define BBCn_PMUQF            ( 0x0082 )
#define BBCn_PMUI             ( 0x0083 )
#define BBCn_PMUQ             ( 0x0084 )
#define BBCn_CNTC             ( 0x0090 )
#define BBCn_CNT0             ( 0x0091 )
#define BBCn_CNT1             ( 0x0092 )
#define BBCn_CNT2             ( 0x0093 )
#define BBCn_CNT3             ( 0x0094 )

#define BBCn_FBRXS            ( 0x0000 )
#define BBCn_FBRXE            ( 0x07FE )
#define BBCn_FBTXS            ( 0x0800 )
#define BBCn_FBTXE            ( 0x0FFE )

/******************************************************************************/

#endif /* AT86RF215_REGS_HPP_ */
